@@ -2,8 +2,12 @@
 #define PERLIO_UTIL_H
 
 #define  PERLIO_FUNCS_CONST
-#include "perliol.h"
 
+#include "EXTERN.h"
+#include "perl.h"
+#include "XSUB.h"
+
+#include "perliol.h"
 
 #define LayerFetch(layer, n) ((layer)->array[n].funcs)
 #define LayerFetchSafe(layer, n) ( ((n) >= 0 && (n) < (layer)->cur) \
